/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 - 2019 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
(function (channel, $, client, ns, logger) {
    'use strict';

    var $editorBody = $('body'),
        statusType = 'siteimprove',
        siStyles = top.window.document.getElementById('siteimprove-highlight-styles'),
        siIframeId = 'siteimprove-contentcheck-content',
        contentCheckIframe = top.window.document.getElementById(siIframeId),
        $siIframe = channel.find("#" + siIframeId),
        $siWrapper = channel.find("#siteimprove-contentcheck-wrapper"),
        $siCheckPageButton = channel.find(".siteimprove-aem-checkPage"),
        $aemContentWrapper = channel.find("#ContentWrapper"),
        contentFrame = ns.ContentFrame,
        $aemScrollView = contentFrame.scrollView,
        siActive = "si-active",
        siEdit = "si-edit",
        siHighlight = "si-highlight",
        siTainted = "si-tainted",
        siEdited = "si-edited",
        inlineElementNames = [
            'a', 'abbr', 'acronym', 'audio',
            'b', 'bdi', 'bdo', 'big', 'br', 'button',
            'canvas', 'cite', 'code',
            'data', 'datalist', 'del', 'dfn',
            'em', 'embed',
            'i', 'iframe', 'img', 'input', 'ins',
            'kbd',
            'label',
            'map', 'mark', 'meter',
            'noscript',
            'object', 'output',
            'picture', 'progress',
            'q',
            'ruby',
            's', 'samp', 'script', 'select', 'slot', 'small', 'span', 'strong', 'sub', 'sup', 'svg',
            'template', 'textarea', 'time',
            'u',
            'tt',
            'var', 'video',
            'wbr'
        ],
        statusPriority = 10000,
        // TODO: ugly global, not really needed as we could just search for .si-highlight elements
        currentHighlightedElement,
        statusBarActions = {
            dismiss : {
                id: 'warning-dismiss',
                label: 'Dismiss'
            },
            goToBlueprint: {
                id: 'go-to-blueprint',
                label: 'Go to blueprint'
            }
        };

    function isSiteimproveActive() {
        return $aemScrollView.hasClass(siActive);
    }

    function markHighlightAsVisited() {
        currentHighlightedElement.addClass(siTainted);
    }

    function markHighlightAsEdited() {
        if (isSiteimproveActive()) {
            if (currentHighlightedElement && currentHighlightedElement.length) {
                currentHighlightedElement.addClass(siEdited);
            }
        }
    }

    function changeSiDisplay(hide) {
        logger.info('changing siteimprove display, hide=', hide);
        if (hide) {
            channel.find(".si-iframe-container").hide();
            channel.find(".si-toggle-container").show();
        } else {
            channel.find(".si-iframe-container").show();
            channel.find(".si-toggle-container").hide();
        }
    }

    function getContentCheckDocument() {
        return contentCheckIframe.contentWindow.document;
    }

    function isInvisible($el) {
        return $el.is(':hidden') || $el.css('visibility') === 'hidden' || $el.css('opacity') == '0';
    }

    function findClosestVisibleElement($el) {
        if (!isInvisible($el)) {
            return $el;
        } else {
            return findClosestVisibleElement($el.parent());
        }
    }

    function getContentCheckSrc() {
        // modern AEM
        if (typeof contentFrame.getContentPath === 'function') {
            return contentFrame.getContentPath() + ".contentcheckrender.html";
        }

        // 6.2
        if (typeof contentFrame.currentLocation === 'function') {
            var loc = contentFrame.currentLocation();
            return loc.substr(0, loc.indexOf(".html")) + ".contentcheckrender.html";
        }

        // stupid fallback
        var path = window.location.pathname;
        path = path.substr(path.indexOf("/editor.html") + "editor.html".length);
        return path.substr(0, path.indexOf(".html")) + ".contentcheckrender.html";
    }

    function isContentCheckViewActive() {
        return $aemScrollView.hasClass(siActive);
    }

    function getContentCheckHtml() {
        return getContentCheckDocument().getElementsByTagName("html")[0].innerHTML;
    }

    function getElementInContentCheckFrame(selector) {
        // possibility of invalid selectors
        return $(getContentCheckDocument()).find(selector);
    }

    function getStatusBar() {
        if (typeof ns.ui.statusBarManager === 'object' && typeof ns.ui.statusBarManager.getStatusBar === 'function') {
            return ns.ui.statusBarManager.getStatusBar()
        }
        logger.debug("Returning fake status bar for older AEM versions");
        // AEM 6.2
        return {
            addStatus: function (type, title, message, shortmessage, priority, variant, icon, data, actions) {
                logger.debug("Fake addstatus, showing AEM alert for old AEM version.");
                $(window).adaptTo("foundation-ui").alert(title, message);
            },
            removeStatus: function(statusId) {
                logger.trace("fake remove status");
            }
        }
    }

    function isLiveCopy(editable) {
        return ns.MsmAuthoringHelper.isLiveCopy(editable);
    }

    function getBlueprintPath(editable) {
        return ns.MsmAuthoringHelper.getBlueprintPath(editable);
    }

    function getMsmSourcePath(editable) {
        return editable.config["msm:liveRelationship"]["msm:sourcePath"];
    }

    function getEditablesByPath(path) {
        logger.info("finding editable for path ", path);
        return ns.editables.find({'path':'' + path + ''});
    }

    function scrollIntoView(domElement) {
        domElement.scrollIntoView({ block: 'center',  behavior: 'smooth' });
    }

    function editEditable($editable) {
        if ($editable.length) {
            var editable = $editable[0];

            logger.info("Got editable", editable);

            // sometimes dom gets lost, investigate
            if (editable.dom) {
                scrollIntoView(editable.dom[0]);
            }

            if (ns.EditorFrame.editableToolbar.config.actions.EDIT.condition(editable)) {
                changeSiDisplay(true);
                $aemScrollView.addClass(siEdit);
                ns.EditorFrame.editableToolbar.config.actions.EDIT.execute(editable);
            } else if (ns.EditorFrame.editableToolbar.config.actions.CONFIGURE.condition(editable)) {
                changeSiDisplay(true);
                $aemScrollView.addClass(siEdit);
                ns.EditorFrame.editableToolbar.config.actions.CONFIGURE.execute(editable);
            } else {
                logger.info("No fitting actions available for component with path", editable.path);
                var message = "No fitting actions available for component '" + editable.overlay.dom.attr('title') + "' with path " + editable.path + ".";
                addStatusWarning(message, editable);
            }
        }
    }

    // fixme: pass in element with data attribute -> this way we can call parent() and check this if not existing.
    function editComponent($closestComponentPathEl) {
        if (!$closestComponentPathEl) {
            logger.info("reached root, cannot find any more component resource paths");
            addStatusWarning("Reached root element, unable to find component for editing");
            return;
        }
        var resourcePath = $closestComponentPathEl.attr('data-si-resource-path');
        if (resourcePath) {
            var editables = getEditablesByPath(resourcePath);
            if (editables.length) {
                logger.info("Found editables by path", editables);
                editEditable(editables);
            } else {
                logger.info("Could not find editables by path, trying parent");
                editComponent(findClosestElementHavingComponentPath($closestComponentPathEl.parent()));
            }
        }
    }

    function openEditableFromUrl() {
        // open editable if url parameter is present
        var url = new URL(window.location.href),
            editablePath = url.searchParams.get("siteimprove-editable");
        if (editablePath && editablePath.length > 0) {
            logger.info("Handling url parameter editablePath", editablePath);
            var editables = getEditablesByPath(editablePath);

            if (editables.length) {
                editEditable(editables);
                logger.debug("Cleaning url parameter editablePath from url");
                window.history.replaceState({}, document.title, url.pathname);
            }
        }
    }

    function isInlineElement(el) {
        return el !== undefined && $.inArray(el.tagName.toLocaleLowerCase(), inlineElementNames) > -1;
    }

    function isTopMostElement(el) {
        var offset = $(el).offset();
        if (offset) {
            return getContentCheckDocument().elementFromPoint(offset.left, offset.top) == el;
        } else {
            // for example invisible elements...
            return true;
        }
    }

    function addStatusWarning(message, editable) {
        var actions = [
                statusBarActions.dismiss
            ],
            data = {},
            statusBar = getStatusBar();

        if (editable) {
            data.editable = editable;
            if (isLiveCopy(editable)) {
                actions.push(statusBarActions.goToBlueprint);
            }
        }

        // API: addStatus(type, title, message [, shortMessage] [, priority] [, variant] [, icon] [, data] [, actions])
        var id = statusBar.addStatus(statusType, "Siteimprove content check", message, "shortmessage", statusPriority, "warning", undefined, data, actions);
        // always show latest status message
        statusBar.showStatus(id);
    }

    function highlightElement($el, isRecursing) {
        //highlight element in place with class
        $el.addClass(siHighlight);
        if (isInvisible($el)) {
            // useful for navigational, e.g. content policy 'fish'
            var $closestComponentPathElement = findClosestElementHavingComponentPath($el),
                closestResourcePath = $closestComponentPathElement.data('si-resource-path'),
                warningMessage = "Element is hidden";
            if (closestResourcePath !== undefined) {
                warningMessage += ", check where to configure '" + $closestComponentPathElement.data('si-resource-path') + "'";
            }
            addStatusWarning(warningMessage);

            // highlight closest visible element
            highlightElement(findClosestVisibleElement($el), true);

            return;

        }
        var el = $el[0];
        scrollIntoView(el);

        // ensure we do not add this multiple times.
        if (!isRecursing && !isTopMostElement(el) && !isInlineElement(el)) {
            logger.info("Appending dialog open helper div to element");
            // not the nicest solution, but useful to evade certain positioning magic
            $('<div class="siteimprove-open-component"><a href="javascript:" class="siteimprove-open-component-link">Edit Component</a></div>')
                .css({
                         "position": "absolute",
                         "left": $el.position().left,
                         "top": $el.position().top,
                         "z-index": 500000
                     }).appendTo($el);
        }

    }

    function findClosestElementHavingComponentPath($el) {
        return $el.closest("[data-si-resource-path]");
    }

    function contentCheckCallback(contentCheckId) {
        if (typeof contentCheckId !== "undefined" && contentCheckId !== null) {
            logger.info('Upload finished successfully for content check id', contentCheckId);
        } else {
            var msg = "We could not run the content check for the given page<br>"
                      + "Please ensure that: "
                      + "<ul>"
                      + "<li>content checking is enabled</li>"
                      + "<li>you are logged into Siteimprove</li>"
                      + "<li>your siteimprove user has access to content check functionality</li>"
                      + "</ul>";
            $(window).adaptTo("foundation-ui").alert("Could not run content check", msg);
        }

    }

    function clearHighlightInfo() {
        // first clear all highlights
        $(getContentCheckDocument()).find('.' + siHighlight).removeClass(siHighlight);
        // then remove all helper divs 'siteimprove-open-component'
        $(getContentCheckDocument()).find('.siteimprove-open-component').remove();
    }

    function contentCheckHighlight(highlightInfo) {

        if (!isSiteimproveActive()) {
            logger.trace("Enforce siteimprove aem overlay to be active for highlighting to work properly");
            $siCheckPageButton.toggleClass("is-selected");
            $aemScrollView.toggleClass(siActive);
        }
        $aemScrollView.removeClass(siEdit);

        clearHighlightInfo();

        logger.debug("Processing highlighting info", highlightInfo);

        $.each(highlightInfo.highlights, function (index) {
            var selector = this.selector,
                offset = this.offset;

            logger.debug("Processing highlight", this);

            if (offset) {
                logger.debug('offset available, highlighting text content using', this);

                try {
                    var doc = getContentCheckDocument(),
                        parentNode = doc.querySelector(selector),
                        oldChildNode = parentNode.childNodes.item(offset.child),
                        newChildNode = doc.createElement("span");

                    newChildNode.innerHTML =
                        oldChildNode.textContent.substr(0, offset.start) + '<span class="si-highlight">'
                        + oldChildNode.textContent.substr(offset.start, offset.length) + '</span>'
                        + oldChildNode.textContent.substr(offset.start + offset.length);

                    parentNode.replaceChild(newChildNode, oldChildNode);

                    scrollIntoView(newChildNode);

                } catch (e) {
                    logger.info(e);
                }
            } else if (selector) {

                if ("BODY" === selector.toUpperCase() || selector.toUpperCase().indexOf("HEAD") > -1) {
                    logger.debug("nothing visible, no point in highlighting the whole document, selector: ", selector.toUpperCase());
                    var message = "This item can not be highlighted in a meaningful way. Please check your page properties.";
                    addStatusWarning(message);
                } else {
                    logger.debug("using selector for highlighting:", selector);
                    var $elToHighlight = getElementInContentCheckFrame(selector);
                    highlightElement($elToHighlight, false);
                }
            }
        });
    }

    function ensureIframeHasSiteimproveContentCheckStyles(contentCheckIframe) {
        var iframeHead = getContentCheckDocument().head;
        if (!iframeHead.querySelector(siStyles.id)) {
            iframeHead.appendChild(siStyles.cloneNode(true));
        }
    }

    function disableIframeInteractions($iframeContents) {
        $iframeContents.find('*').off();
        $iframeContents.on('click', 'a,button', function(event) {
            event.preventDefault();
            event.stopPropagation();
        });
    }

    function addHighlightInteractions($iframeContents) {
        $iframeContents.on('click', '.' + siHighlight, function () {
            currentHighlightedElement = $(this);
            logger.debug("Handling click on highlighted element", currentHighlightedElement);
            markHighlightAsVisited();

            var $closestComponentPathEl = findClosestElementHavingComponentPath($(this));
            editComponent($closestComponentPathEl)
        });

        $iframeContents.on('click', '.siteimprove-open-component-link', function () {
            $(this).closest('.' + siHighlight).trigger('click');
        });
    }

    $siIframe.on("load", function () {
        var $this = $(this),
            contents = $this.contents(),
            href = contents.get(0).location.href;

        if (href !== '' && href !== 'about:blank') {
            logger.info("Loading content check iframe from", href);
            ensureIframeHasSiteimproveContentCheckStyles(contentCheckIframe);
            disableIframeInteractions(contents);
            addHighlightInteractions(contents);

            // trigger check, does not necessarily execute if the user is missing permissions :<
            client.siCheckHtml(getContentCheckHtml(), contentCheckCallback, contentCheckHighlight);
        }

    });

    // hook for content check api -> probably better to create an ID for the button in the header toolbar
    $editorBody.on('click', '.siteimprove-aem-checkPage', function() {
        var src = getContentCheckSrc(),
            prevSrc = $siIframe.attr("src");

        if (ns.SidePanel) {
            if (!$siCheckPageButton.hasClass("is-selected")) {
                ns.SidePanel.close();
            }
        }

        $siCheckPageButton.toggleClass("is-selected");
        $aemScrollView.toggleClass(siActive);

        // only re-set src if change detected
        if (src !== prevSrc) {
            $siIframe.attr("src", src);
        }
    });


    channel.on("cq-layer-activated", function (event) {
        if (event.layer === "Edit") {
            logger.info("enable siteimprove button by removing disabled attr as we are in layer 'Edit'");
            $siCheckPageButton.removeAttr("disabled");
        } else {
            // disable button by adding disabled attr and hide frame, disabled button is hidden by OOTB css
            logger.trace("Other layer ", event.layer);
            $siCheckPageButton.attr("disabled", "disabled");

            $aemScrollView.removeClass(siActive);
            $siCheckPageButton.removeClass("is-selected");
        }
    });

    // for deep linking into blueprint components
    channel.on("cq-editor-loaded", openEditableFromUrl);

    // interesting event sources:
    // /libs/cq/gui/components/authoring/editors/clientlibs/core/js/ContentFrame.js

    // not sure if we need this as the other event should be fired anyway
    channel.on("cq-content-frame-loaded", function(event) {
        // once per page/navigation
        $siWrapper.css("height", $aemContentWrapper.css("height"));
    });

    // ensure siteimprove frame matches edit frame height
    channel.on("cq-contentframe-layout-change", function () {
        // any layout change like resize or adding/removing components
        var siHeight = $siWrapper.css("height"),
            aemHeight = $aemContentWrapper.css("height");
        if (siHeight !== aemHeight) {
            $siWrapper.css("height", aemHeight);
        }
    });

    channel.on("cq-content-frame-unload", function () {
        $siIframe.attr("src", "");
        $aemScrollView.removeClass(siActive);
    });

    // EditorFrame events
    // /libs/cq/gui/components/authoring/editors/clientlibs/core/js/EditorFrame.js
    channel.on("cq-editor-content-location-changed", function() {
        if (isContentCheckViewActive()) {
            logger.info("Updating iframe location to", getContentCheckSrc());
            $siIframe.attr("src", getContentCheckSrc());
        }
    });


    // statusbar message handling
    channel.on('click', ".editor-StatusBar-action", function() {
        var $this = $(this),
            type = $this.data('statusType'),
            actionId = $this.data('statusActionId'),
            statusData = $this.data('statusData');
        if (type === statusType) {
            if (actionId === statusBarActions.dismiss.id) {
                getStatusBar().removeStatus($this.data('statusId'));
            } else if (actionId === statusBarActions.goToBlueprint.id && statusData.editable) {
                window.open("/editor.html" + getBlueprintPath(statusData.editable)
                            + ".html?siteimprove-editable=" + getMsmSourcePath(statusData.editable));
            }
        }
    });

    // this triggers for both cancel and submit
    channel.on("dialog-closed", function() {
        logger.trace("received dialog-closed event");
        // second
        if (isSiteimproveActive()) {
            $aemScrollView.removeClass(siEdit);
            changeSiDisplay(false);
            currentHighlightedElement = undefined;
        }
    });

    channel.on("inline-edit-finish", function () {
        logger.trace("received inline-edit-finish event");
        if (isSiteimproveActive()) {
            $aemScrollView.removeClass(siEdit);
            changeSiDisplay(false);
        }
    });

    channel.on("inline-edit-cancel", function () {
        logger.trace("received inline-edit-cancel event");
        if (isSiteimproveActive()) {
            $aemScrollView.removeClass(siEdit);
            changeSiDisplay(false);
        }
    });

    // fired when submitting dialog or pressing enter in inplace or setting focus somewhere else when in place
    // basically we assume that the component has been edited
    channel.on("cq-persistence-after-update", function(event) {
        logger.trace("received cq-persistence-after-update event");
        markHighlightAsEdited();
    });

}(jQuery(document), Granite.$, window.siteimprove.aem, Granite.author, window.siteimprove.logger));
