/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 - 2019 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
window.siteimprove = window.siteimprove || {};
window.siteimprove.logger = window.siteimprove.logger || (function (channel) {
    "use strict";

    var loggerSettingKey = 'siteimprove.logger.threshold',
        loggerOff = '99',
        threshold = localStorage.getItem(loggerSettingKey) != null ? localStorage.getItem(loggerSettingKey) : loggerOff,
        loglevel = {
            trace: {
                severity: 1,
                label: 'TRACE',
                logLabel: '[TRACE]'
            },
            debug: {
                severity: 2,
                label: 'DEBUG',
                logLabel: '[DEBUG]'
            },
            info: {
                severity: 3,
                label: 'INFO',
                logLabel: '[INFO] '
            },
            warn: {
                severity: 4,
                label: 'WARN',
                logLabel: '[WARN] '
            },
            error: {
                severity: 5,
                label: 'ERROR',
                logLabel: '[ERROR]'
            }
        },
        log = function (args, loglevel) {
            if (typeof console === "undefined") {
                return function () {
                };
            }

            if (loglevel.severity >= threshold) {
                var messages = Array.prototype.slice.call(args);
                messages.unshift(new Date().toISOString(), loglevel.logLabel, "[Siteimprove]");

                Function.prototype.apply.call(console.log, console, messages);
            }
        },
        error = function () {
            log(arguments, loglevel.error);
        },
        warn = function () {
            log(arguments, loglevel.warn);
        },
        info = function () {
            log(arguments, loglevel.info);
        },
        debug = function () {
            log(arguments, loglevel.debug);
        },
        trace = function () {
            log(arguments, loglevel.trace);
        },
        /**
         * Sets the logging threshold, e.g. setting 'debug' includes 'error', 'warn', 'info' and 'debug' level itself.
         * @param label one of 'error', 'warn', 'info', 'debug', 'trace'. If an invalid label is given the logging is disabled.
         */
        setLogThreshold = function (label) {
            if (loglevel[label]) {
                localStorage.setItem(loggerSettingKey, loglevel[label].severity);
            } else {
                localStorage.setItem(loggerSettingKey, loggerOff);
            }
        };

    // public functions
    return {
        error: error,
        warn: warn,
        info: info,
        debug: debug,
        trace: trace,
        setLogThreshold: setLogThreshold
    }

})(window);