/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 - 2019 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
(function(window, $, client, Handlebars, logger) {

    window.siteimprove = window.siteimprove || {};

    window.siteimprove.conditions = window.siteimprove.conditions || {};

    window.siteimprove.conditions.canContentCheck = function(name, el, config, collection, selections) {
        logger.trace('checking if canContentCheck');
        // TODO: use provisioning information for this
        return true;
        /*
        return client.getConfig($(selections).data("foundation-collection-item-id"))
            .done(function(config) {
                return config.canPrePublishCheck;
            })
            .fail(function(){return false}); */
    };

    window.siteimprove.conditions.canUnpublishCheck = function(name, el, config, collection, selections) {
        // TODO: use provisioning information for this
        logger.trace('checking if canUnpublishCheck');
        return true;
        /*
            return client.getConfig($(selections).data("foundation-collection-item-id"))
                .done(function(config) {
                    logger.info("done " + config.canUnpublishCheck);
                    return config.canUnpublishCheck;
                })
                .fail(function(){
                    logger.info("fail");
                    return false
                }); */
    };

    var registry = $(window).adaptTo('foundation-registry');

    registry.register('foundation.collection.action.activecondition', {
        name: 'siteimprove.aem.check.unpublish.isActive',
        handler: siteimprove.conditions.canUnpublishCheck
    });

    registry.register('foundation.collection.action.activecondition', {
        name: 'siteimprove.aem.check.contentcheck.isActive',
        handler: siteimprove.conditions.canContentCheck
    });

    // TODO: find a better place to place this dictionary
    // TODO: switch to Granite.I18n.get("...");
    var dict = {
        // analytics
        "external_referring_pages": "External referring pages within 30 days",
        "has_behaviour_map": "Active Behaviour map",
        "page_views": "Page views within 30 days",
        "potential_campaigns": "Part of a Monitor Campaign",
        "potential_key_metrics": "Part of a Key Metrics configuration",

        // qa
        "documents_on_page": "Documents present on this page",
        "documents_only_on_this_page": "Documents only present on this page",
        "internal_referring_pages": "Internal pages referring to this page",

        // seo
        "backlinks": "Backlinks to this page (up to 5)",
        "is_target_page": "Is or has this page been a targeted page",
        "ranking_keywords": "Does this page rank on Google",
    };

    function createTab(tablist, panelstack, dataArray, section, selected, $container) {
        var tableMarkup = '<h3>{{label}}</h3>'
                          + '<table is="coral-table" style="{{style}}">'
                          + '  <colgroup>'
                          + '    <col is="coral-table-column" sortable sortabledirection="ascending">'
                          + '    <col is="coral-table-column" sortable>'
                          + '  </colgroup>'
                          + '  <thead is="coral-table-head" sticky>'
                          + '    <tr is="coral-table-row">'
                          + '      <th is="coral-table-headercell">Title</th>'
                          + '      <th is="coral-table-headercell">Url</th>'
                          + '    </tr>'
                          + '  </thead>'
                          + '  <tbody is="coral-table-body" divider="cell">'
                          + '{{#each rows}}'
                          +    '<tr is="coral-table-row">'
                          + '      <td is="coral-table-cell">{{this.title}}</td>'
                          + '      <td is="coral-table-cell"><a href="{{this.url}}" target="_blank">{{this.url}}</a></td>'
                          + '    </tr>'
                          + '{{/each}}'
                          + '  </tbody>'
                          + '</table>',
            tableTemplate = Handlebars.compile(tableMarkup),
            htmlToRender = '';

        $.each(dataArray, function (key, value) {
            var label = dict[key] || key;
            if (Array.isArray(value)) {
                // only render non empty arrays
                var numberOfItems = value.length;
                if (numberOfItems) {
                    var context = {label: label, rows: value};
                    if (numberOfItems > 5) {
                        context.style = 'height:200px;'
                    }
                    htmlToRender += tableTemplate(context);
                }
            } else {
                htmlToRender += '<h3>' + label +'</h3><div>' + value +'</div>';
            }
        });

        // ensure we create no empty tabs
        if (htmlToRender.length) {
            tablist.items.add(
                {
                    label: {
                        innerHTML: section
                    },
                    hidden: false,
                    selected: selected
                }
            );

            panelstack.items.add(
                {
                    content: {
                        innerHTML: htmlToRender
                    }
                }
            );
        }

    }

    registry.register("foundation.collection.action.action", {
        name: 'siteimprove.aem.check.unpublish',
        handler: function(name, el, config, collection, selections) {
            var collectionItem = $(selections).data("foundation-collection-item-id");
            client.unpublishCheck(collectionItem)
                .always(function(data) {
                    // promise return object is empty if something went wrong, url is mandatory field
                    if (data.url) {

                        // TODO: convert to handlebars
                        // TODO: also include the tab stuff in this template.
                        // TODO: collect everything for context and template rendering

                        var $html = $('<coral-dialog id="siteimprove-unpublish-dialog">' +
                                   '  <coral-dialog-header>'
                                      + '<svg xmlns="http://www.w3.org/2000/svg" style="position: relative; top: 4px; margin-right:5px;" height="22px" width="22px" viewBox="0 0 80 80">'
                                      + '<path d="M40 0C18 0 0 18 0 40.1 0 62.1 18 80 40 80 62 80 80 62.1 80 40.1 80 18 62.1 0 40 0Zm0 67C25.2 67 13.1 54.9 13.1 40.1 13.1 25.2 25.2 13.2 40 13.2c14.4 0 26.2 11.4 26.9 25.6-16.7 12-30.5-10.9-46.5-2.6 18.7-5.6 25.1 22.3 43.7 16C59.6 60.9 50.5 67 40 67Z" fill="#3d495f"/>'
                                      + '</svg>'
                                      + 'Siteimprove Unpublish Check'
                                      + '</coral-dialog-header>' +
                                      // fixed dimensions forced to not have the dialog jump in case of different content sizes
                                   '  <coral-dialog-content style="width:800px;height:550px;">'
                                      + '<h3>General Information</h3>'
                                      + '<strong>Page Title: </strong>' + data.title + '<br>'
                                      + '<strong>External url: </strong><a href="' + data.url + '" target="_blank">' + data.url + '</a><br>'
                                      + '<h3>Relations</h3>'
                                      + '<div id="siteimprove-unpublish-dialog-content"></div>'
                                      + '</coral-dialog-content>' +
                                   '  <coral-dialog-footer>' +
                                        '<button is="coral-button" variant="primary" coral-close="" class="coral-Button coral-Button--primary" size="M">' +
                                          '<coral-button-label>Close</coral-button-label>' +
                                        '</button>' +
                                      '</coral-dialog-footer>' +
                                   '</coral-dialog>'),
                            $container = $html.find("#siteimprove-unpublish-dialog-content"),
                            tablist = new Coral.TabList().set({
                                                                  target: '#siteimprove-unpublish-panel'
                                                              }),
                            panelstack = new Coral.PanelStack().set({
                                                                        id: 'siteimprove-unpublish-panel'
                                                                    });

                        createTab(tablist, panelstack, data.relations.quality_assurance, 'QA', true, $container);
                        createTab(tablist, panelstack, data.relations.seov2, 'SEO', false, $container);
                        createTab(tablist, panelstack, data.relations.analytics, 'Analytics', false, $container);

                        var tabview = new Coral.TabView().set({
                                                              panelStack: panelstack,
                                                              tabList: tablist
                                                          });
                        $container.append(tabview);

                        $html.appendTo(document.body);

                        var dialog = document.querySelector('#siteimprove-unpublish-dialog');

                        // cleanup dialog for GC after closing
                        dialog.on('coral-overlay:close', function(event) {
                            logger.trace("Closing dialog and cleaning up");
                            dialog.remove();
                        });

                        // dialog is not properly centered, BUT size jumps with different tabs..
                        // centering also puts the dialog BELOW the toolbar in sites.html...
                        dialog.on('coral-overlay:open', function(event) {
                            // too late, jumping
                            //dialog.querySelector('.coral3-Dialog-wrapper').style.cssText = 'position: absolute; top: 10%; left: 25%; width: 50%';
                            logger.trace("Centering dialog");
                            dialog.center();
                        });

                        dialog.show();

                    } else {
                        var msg = "We could not retrieve any Unpublish Check information for page " + collectionItem + ". <br>"
                                  + "Please ensure that: "
                                  + "<ul>"
                                  + "<li>the cloud configuration is applied to the page or its parent</li>"
                                  + "<li>the API user has been set up correctly</li>"
                                  + "<li>the page has been published</li>"
                                  + "</ul><br>";

                        if (data.errorMessage) {
                            msg += "Error message: " + data.errorMessage;
                        }

                        $(window).adaptTo("foundation-ui").alert("No Unpublish Check results", msg);
                    }
                });

            // TODO: do not bind this to sites.html only, this is usable in editor.html as well
        }
    });

}(window, window.Granite.$, window.siteimprove.aem, Handlebars, window.siteimprove.logger));
