/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 - 2019 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
CQ.Ext.ns("SiteimproveAEM.CloudConfig");

(function (channel, $, client, ns, logger) {

    var ENDPOINT = 'https://api.siteimprove.com/v2/settings/content_checking',
        getAuthorization = function () {
            var usernameField = CQ.Ext.getCmp("contentCheckUsername"),
                passwordField = CQ.Ext.getCmp("contentCheckPassword"),
                username = usernameField ? usernameField.getValue() : "",
                password = passwordField ? passwordField.getValue() : "";

            if (username && password) {
                return btoa(username + ':' + password);
            }

            return '';
        },
        getStatus = function () {

            function sucessGetStatus(data) {
                logger.debug("sucessGetStatus", data);
                return data;
            }

            function failGetStatus(jqXHR) {
                logger.warn("failGetStatus", jqXHR);
                return transformFailure(jqXHR);
            }

            return $.ajax(
                {
                    headers: {'Authorization': 'Basic ' + getAuthorization()},
                    method: 'GET',
                    url: ENDPOINT
                }).then(sucessGetStatus, failGetStatus)
                .always(function (data) {
                    logger.debug("always get", data);
                    return data;
                });
        },

        transformFailure = function (jqXHR) {
            return {
                statusCode: jqXHR.status,
                statusText: jqXHR.statusText,
                message: jqXHR.responseJSON && jqXHR.responseJSON.message ? jqXHR.responseJSON.message : jqXHR.statusText
            }
        },

        getStatusTextArea = function () {
            return CQ.Ext.getCmp("contentCheckStatus");
        },
        setStatusText = function (text) {
            getStatusTextArea().setValue(text);
        },

        appendStatusText = function (text) {
            var textArea = getStatusTextArea(),
                oldText = textArea.getValue();
            textArea.setValue(oldText + text);
        };

    ns.enableContentChecking = function () {

        function successEnableContentChecking(data) {
            logger.info("successEnableContentChecking", data);

            setStatusText('Polling content check provisioning status');

            var pollHandle = window.setInterval(function () {

                getStatus().always(function (data) {
                    if (data.is_ready) {
                        window.clearInterval(pollHandle);
                        setStatusText("Content checking provisioned");

                    } else {
                        appendStatusText('.');
                    }
                });

            }, 1000);

            return data;
        }

        function failEnableContentChecking(jqXHR) {
            logger.warn("failEnableContentChecking", jqXHR);
            CQ.Ext.Msg.alert("Content Check Provisioning failed", "Could not provision content checking. " + JSON.stringify(jqXHR));
            return transformFailure(jqXHR);
        }

        return $.ajax(
            {
                headers: {'Authorization': 'Basic ' + getAuthorization()},
                contentType: 'application/json',
                method: 'POST',
                url: ENDPOINT
            }).then(successEnableContentChecking, failEnableContentChecking)
            .always(function (data) {
                logger.info("always post", data);
                return data;
            });
    };

    ns.loadContent = function () {

        logger.debug("Activate cloud config dialog");
        try {
            logger.debug("Have username and password, checking status");

            getStatus().always(function (data) {
                setStatusText(JSON.stringify(data));

                if (!data.is_ready) {
                    logger.info('No settings available, need to provision content checking');
                    ns.enableContentChecking();
                } else {
                    logger.debug('Content checking is ready.');
                }
            });
        } catch (e) {
            logger.info(e);
        }

    };

    ns.verifyConfig = function () {
        getStatus().always(function (data) {
            CQ.Ext.Msg.alert("Content Check Status", "Content check provisioned: " + data.is_ready);
        });
    };

})(window, Granite.$, window.siteimprove.aem, SiteimproveAEM.CloudConfig, window.siteimprove.logger);

