/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 - 2019 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
/**
 * Source: http://experience-aem.blogspot.com/2015/06/aem-61-classic-ui-composite-multifield-storing-values-as-nodes.html <br>
 * Adapted to create same structure as Granite UI multifield.
 */
CQ.Ext.ns("SiteimproveAEM");

SiteimproveAEM.MultiFieldPanel = CQ.Ext.extend(CQ.Ext.Panel, {
    constructor: function(config){
        config = config || {};
        SiteimproveAEM.MultiFieldPanel.superclass.constructor.call(this, config);
    },

    initComponent: function () {
        SiteimproveAEM.MultiFieldPanel.superclass.initComponent.call(this);

        function addName(items, prefix, counter){
            items.each(function(i){
                if(!i.hasOwnProperty("dName")){
                    return;
                }

                i.name = prefix + "/item" + (counter) + "/" + i.dName;

                //workaround for xtype select
                if(i.hiddenField){
                    i.hiddenField.name = prefix + "/item" + (counter) + "/" + i.dName;
                }

                if(i.el && i.el.dom){ //form serialization workaround
                    i.el.dom.name = prefix + "/item" + (counter) + "/" + i.dName;
                }
            },this);
        }

        var multi = this.findParentByType("multifield"),
            multiPanels = multi.findByType("siteimprove-multi-field-panel");

        addName(this.items, this.name, multiPanels.length);

        multi.on("removeditem", function(){
            // FIXME: this bugs a lot with moving/deleting items
            multiPanels = multi.findByType("siteimprove-multi-field-panel");

            for(var x = 0; x < multiPanels.length; x++){
                addName(multiPanels[x].items, multiPanels[x].name, x);
            }
        });

        // ensure proper rendering of multifield
        var tabPanel = multi.findParentByType("tabpanel");
        if(tabPanel){
            tabPanel.on("tabchange", function(panel){
                panel.doLayout();
            });
        }
    },

    afterRender : function(){
        SiteimproveAEM.MultiFieldPanel.superclass.afterRender.call(this);

        this.items.each(function(){
            if(!this.contentBasedOptionsURL
               || this.contentBasedOptionsURL.indexOf(CQ.form.Selection.PATH_PLACEHOLDER) < 0){
                return;
            }

            this.processPath(this.findParentByType('dialog').path);
        })
    },

    getValue: function () {
        var pData = {};

        this.items.each(function(i){
            if(!i.hasOwnProperty("dName")){
                return;
            }

            pData[i.dName] = i.getValue();
        });

        return pData;
    },

    setValue: function (value) {
        var counter = 0, item,
            multi = this.findParentByType("multifield"),
            multiPanels = multi.findByType("siteimprove-multi-field-panel");

        // called for each entry of the multifield
        if(multiPanels.length == 1){
            var key = "item" + counter;
            item = value[key];
        }else{
            item = value;
        }

        // basically the select and the threshold field
        // setting the first field
        this.items.each(function(i){
            if(!i.hasOwnProperty("dName")){
                return;
            }

            i.setValue(item[i.dName]);
        });


        if(multiPanels.length == 1){
            while(true){
                item = value["item" + (++counter)];

                if(!item){
                    break;
                }

                multi.addItem(item);
            }
        }
    },

    validate: function(){
        return true;
    },

    getName: function(){
        return this.name;
    }
});

CQ.Ext.reg("siteimprove-multi-field-panel", SiteimproveAEM.MultiFieldPanel);
